@@ -16,7 +16,6 @@ import os
 from unittest import mock
 
 import ddt
-import six
 
 from os_brick import exception
 from os_brick.initiator.connectors import base
@@ -61,6 +60,32 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
                  'vport_create': '<store method only>',
                  'vport_delete': '<store method only>'}]
 
+    def fake_get_fc_hbas_with_platform(self):
+        return [{'ClassDevice': 'host1',
+                 'ClassDevicePath': '/sys/devices/platform/smb'
+                                    '/smb:motherboard/80040000000.peu0-c0'
+                                    '/pci0000:00/0000:00:03.0'
+                                    '/0000:05:00.2/host1/fc_host/host1',
+                 'dev_loss_tmo': '30',
+                 'fabric_name': '0x1000000533f55566',
+                 'issue_lip': '<store method only>',
+                 'max_npiv_vports': '255',
+                 'maxframe_size': '2048 bytes',
+                 'node_name': '0x200010604b019419',
+                 'npiv_vports_inuse': '0',
+                 'port_id': '0x680409',
+                 'port_name': '0x100010604b019419',
+                 'port_state': 'Online',
+                 'port_type': 'NPort (fabric via point-to-point)',
+                 'speed': '10 Gbit',
+                 'supported_classes': 'Class 3',
+                 'supported_speeds': '10 Gbit',
+                 'symbolic_name': 'Emulex 554M FV4.0.493.0 DV8.3.27',
+                 'tgtid_bind_type': 'wwpn (World Wide Port Name)',
+                 'uevent': None,
+                 'vport_create': '<store method only>',
+                 'vport_delete': '<store method only>'}]
+
     def fake_get_fc_hbas_info(self):
         hbas = self.fake_get_fc_hbas()
         info = [{'port_name': hbas[0]['port_name'].replace('0x', ''),
@@ -69,6 +94,14 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
                  'device_path': hbas[0]['ClassDevicePath']}]
         return info
 
+    def fake_get_fc_hbas_info_with_platform(self):
+        hbas = self.fake_get_fc_hbas_with_platform()
+        info = [{'port_name': hbas[0]['port_name'].replace('0x', ''),
+                 'node_name': hbas[0]['node_name'].replace('0x', ''),
+                 'host_device': hbas[0]['ClassDevice'],
+                 'device_path': hbas[0]['ClassDevicePath']}]
+        return info
+
     def fibrechan_connection(self, volume, location, wwn, lun=1):
         return {'driver_volume_type': 'fibrechan',
                 'data': {
@@ -100,19 +133,45 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
     def test_get_pci_num(self):
         hba = {'device_path': "/sys/devices/pci0000:00/0000:00:03.0"
                               "/0000:05:00.3/host2/fc_host/host2"}
-        pci_num = self.connector._get_pci_num(hba)
+        platform, pci_num = self.connector._get_pci_num(hba)
         self.assertEqual("0000:05:00.3", pci_num)
+        self.assertIsNone(platform)
 
         hba = {'device_path': "/sys/devices/pci0000:00/0000:00:03.0"
                               "/0000:05:00.3/0000:06:00.6/host2/fc_host/host2"}
-        pci_num = self.connector._get_pci_num(hba)
+        platform, pci_num = self.connector._get_pci_num(hba)
         self.assertEqual("0000:06:00.6", pci_num)
+        self.assertIsNone(platform)
 
         hba = {'device_path': "/sys/devices/pci0000:20/0000:20:03.0"
                               "/0000:21:00.2/net/ens2f2/ctlr_2/host3"
                               "/fc_host/host3"}
-        pci_num = self.connector._get_pci_num(hba)
+        platform, pci_num = self.connector._get_pci_num(hba)
         self.assertEqual("0000:21:00.2", pci_num)
+        self.assertIsNone(platform)
+
+    def test_get_pci_num_with_platform(self):
+        hba = {'device_path': "/sys/devices/platform/smb/smb:motherboard/"
+                              "80040000000.peu0-c0/pci0000:00/0000:00:03.0"
+                              "/0000:05:00.3/host2/fc_host/host2"}
+        platform, pci_num = self.connector._get_pci_num(hba)
+        self.assertEqual("0000:05:00.3", pci_num)
+        self.assertEqual("platform-80040000000.peu0-c0", platform)
+
+        hba = {'device_path': "/sys/devices/platform/smb/smb:motherboard"
+                              "/80040000000.peu0-c0/pci0000:00/0000:00:03.0"
+                              "/0000:05:00.3/0000:06:00.6/host2/fc_host/host2"}
+        platform, pci_num = self.connector._get_pci_num(hba)
+        self.assertEqual("0000:06:00.6", pci_num)
+        self.assertEqual("platform-80040000000.peu0-c0", platform)
+
+        hba = {'device_path': "/sys/devices/platform/smb"
+                              "/smb:motherboard/80040000000.peu0-c0/pci0000:20"
+                              "/0000:20:03.0/0000:21:00.2"
+                              "/net/ens2f2/ctlr_2/host3/fc_host/host3"}
+        platform, pci_num = self.connector._get_pci_num(hba)
+        self.assertEqual("0000:21:00.2", pci_num)
+        self.assertEqual("platform-80040000000.peu0-c0", platform)
 
     @mock.patch.object(os.path, 'exists', return_value=True)
     @mock.patch.object(linuxfc.LinuxFibreChannel, 'get_fc_hbas')
@@ -136,6 +195,30 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
                     '-fc-0x1234567890123456-lun-1']
         self.assertEqual(expected, volume_paths)
 
+    @mock.patch.object(os.path, 'exists', return_value=True)
+    @mock.patch.object(linuxfc.LinuxFibreChannel, 'get_fc_hbas')
+    @mock.patch.object(linuxfc.LinuxFibreChannel, 'get_fc_hbas_info')
+    def test_get_volume_paths_with_platform(self, fake_fc_hbas_info,
+                                            fake_fc_hbas, fake_exists):
+        fake_fc_hbas.side_effect = self.fake_get_fc_hbas_with_platform
+        fake_fc_hbas_info.side_effect \
+            = self.fake_get_fc_hbas_info_with_platform
+
+        name = 'volume-00000001'
+        vol = {'id': 1, 'name': name}
+        location = '10.0.2.15:3260'
+        wwn = '1234567890123456'
+        connection_info = self.fibrechan_connection(vol, location, wwn)
+        conn_data = self.connector._add_targets_to_connection_properties(
+            connection_info['data']
+        )
+        volume_paths = self.connector.get_volume_paths(conn_data)
+
+        expected = ['/dev/disk/by-path'
+                    '/platform-80040000000.peu0-c0-pci-0000:05:00.2'
+                    '-fc-0x1234567890123456-lun-1']
+        self.assertEqual(expected, volume_paths)
+
     @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_rw')
     @mock.patch.object(os.path, 'exists', return_value=True)
     @mock.patch.object(os.path, 'realpath', return_value='/dev/sdb')
@@ -175,7 +258,7 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
         # Should work for string, unicode, and list
         wwns_luns = [
             ('1234567890123456', 1),
-            (six.text_type('1234567890123456'), 1),
+            (str('1234567890123456'), 1),
             (['1234567890123456', '1234567890123457'], 1),
             (['1234567890123456', '1234567890123457'], 1),
         ]
@@ -391,6 +474,9 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
             find_mp_dev_mock, 'rw', False)
 
         self.assertNotIn('multipathd_id', connection_info['data'])
+        # Ensure we don't call it with the real path
+        device_name = discover_mp_dev_mock.call_args[0][-1]
+        self.assertNotEqual(realpath_mock.return_value, device_name)
 
     @mock.patch.object(fibre_channel.FibreChannelConnector, 'get_volume_paths')
     def test_extend_volume_no_path(self, mock_volume_paths):
@@ -636,7 +722,7 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
         # Check that we turn to lowercase target wwns
         key = 'target_wwns' if 'target_wwns' in target_info else 'target_wwn'
         wwns = target_info.get(key)
-        wwns = [wwns] if isinstance(wwns, six.string_types) else wwns
+        wwns = [wwns] if isinstance(wwns, str) else wwns
         wwns = [w.lower() for w in wwns]
         if wwns:
             self.assertEqual(wwns, conn['data'][key])
@@ -648,17 +734,22 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
 
     @ddt.data(('/dev/mapper/<WWN>', True),
               ('/dev/mapper/mpath0', True),
+              # Check real devices are properly detected as non multipaths
+              ('/dev/sda', False),
               ('/dev/disk/by-path/pci-1-fc-1-lun-1', False))
     @ddt.unpack
     @mock.patch('os_brick.initiator.linuxscsi.LinuxSCSI.remove_scsi_device')
     @mock.patch('os_brick.initiator.linuxscsi.LinuxSCSI.requires_flush')
-    @mock.patch('os_brick.initiator.linuxscsi.LinuxSCSI.get_dev_path')
+    @mock.patch('os_brick.utils.get_dev_path')
     def test__remove_devices(self, path_used, was_multipath, get_dev_path_mock,
                              flush_mock, remove_mock):
+        exc = exception.ExceptionChainer()
         get_dev_path_mock.return_value = path_used
         self.connector._remove_devices(mock.sentinel.con_props,
                                        [{'device': '/dev/sda'}],
-                                       mock.sentinel.device_info)
+                                       mock.sentinel.device_info,
+                                       force=False, exc=exc)
+        self.assertFalse(bool(exc))
         get_dev_path_mock.assert_called_once_with(mock.sentinel.con_props,
                                                   mock.sentinel.device_info)
         flush_mock.assert_called_once_with('/dev/sda', path_used,
@@ -666,6 +757,39 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
         remove_mock.assert_called_once_with('/dev/sda',
                                             flush=flush_mock.return_value)
 
+    @ddt.data(('/dev/mapper/<WWN>', True),
+              ('/dev/mapper/mpath0', True),
+              # Check real devices are properly detected as non multipaths
+              ('/dev/sda', False),
+              ('/dev/disk/by-path/pci-1-fc-1-lun-1', False))
+    @ddt.unpack
+    @mock.patch('os_brick.initiator.linuxscsi.LinuxSCSI.remove_scsi_device')
+    @mock.patch('os_brick.initiator.linuxscsi.LinuxSCSI.requires_flush')
+    @mock.patch('os_brick.utils.get_dev_path')
+    def test__remove_devices_fails(self, path_used, was_multipath,
+                                   get_dev_path_mock, flush_mock, remove_mock):
+        exc = exception.ExceptionChainer()
+        get_dev_path_mock.return_value = path_used
+        remove_mock.side_effect = Exception
+        self.connector._remove_devices(mock.sentinel.con_props,
+                                       [{'device': '/dev/sda'},
+                                        {'device': '/dev/sdb'}],
+                                       mock.sentinel.device_info,
+                                       force=True, exc=exc)
+        self.assertTrue(bool(exc))
+        get_dev_path_mock.assert_called_once_with(mock.sentinel.con_props,
+                                                  mock.sentinel.device_info)
+
+        expect_flush = [mock.call('/dev/sda', path_used, was_multipath),
+                        mock.call('/dev/sdb', path_used, was_multipath)]
+        self.assertEqual(len(expect_flush), flush_mock.call_count)
+        flush_mock.assert_has_calls(expect_flush)
+
+        expect_remove = [mock.call('/dev/sda', flush=flush_mock.return_value),
+                         mock.call('/dev/sdb', flush=flush_mock.return_value)]
+        self.assertEqual(len(expect_remove), remove_mock.call_count)
+        remove_mock.assert_has_calls(expect_remove)
+
     @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_rw')
     @mock.patch.object(os.path, 'exists', return_value=True)
@@ -727,3 +851,94 @@ class FibreChannelConnectorTestCase(test_connector.ConnectorTestCase):
             'tee -a /sys/block/sdc/device/delete',
         ]
         self.assertEqual(expected_commands, self.cmds)
+
+    @ddt.data((False, Exception), (True, Exception), (False, None))
+    @ddt.unpack
+    @mock.patch.object(fibre_channel.FibreChannelConnector, '_remove_devices')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'multipath_del_map')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'flush_multipath_device')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'wait_for_rw')
+    @mock.patch.object(os.path, 'exists', return_value=True)
+    @mock.patch.object(os.path, 'realpath')
+    @mock.patch.object(linuxfc.LinuxFibreChannel, 'get_fc_hbas')
+    @mock.patch.object(linuxfc.LinuxFibreChannel, 'get_fc_hbas_info')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device_path')
+    @mock.patch.object(base.BaseLinuxConnector, 'check_valid_device')
+    def test_disconnect_volume_fails(self, ignore_exc, side_effect,
+                                     check_valid_device_mock,
+                                     find_mp_device_path_mock,
+                                     get_device_info_mock,
+                                     get_scsi_wwn_mock,
+                                     get_fc_hbas_info_mock,
+                                     get_fc_hbas_mock,
+                                     realpath_mock,
+                                     exists_mock,
+                                     wait_for_rw_mock,
+                                     flush_mock,
+                                     del_map_mock,
+                                     remove_mock):
+
+        flush_mock.side_effect = side_effect
+        del_map_mock.side_effect = side_effect
+
+        check_valid_device_mock.return_value = True
+        self.connector.use_multipath = True
+        get_fc_hbas_mock.side_effect = self.fake_get_fc_hbas
+        get_fc_hbas_info_mock.side_effect = self.fake_get_fc_hbas_info
+
+        wwn = '360002ac00000000000000b860000741c'
+        multipath_devname = f'/dev/disk/by-id/dm-uuid-mpath-{wwn}'
+        realpath_mock.return_value = '/dev/dm-1'
+        devices = {"device": multipath_devname,
+                   "id": wwn,
+                   "devices": [{'device': '/dev/sdb',
+                                'address': '1:0:0:1',
+                                'host': 1, 'channel': 0,
+                                'id': 0, 'lun': 1},
+                               {'device': '/dev/sdc',
+                                'address': '1:0:0:2',
+                                'host': 1, 'channel': 0,
+                                'id': 0, 'lun': 1}]}
+        get_device_info_mock.side_effect = devices['devices']
+        get_scsi_wwn_mock.return_value = wwn
+
+        location = '10.0.2.15:3260'
+        name = 'volume-00000001'
+        vol = {'id': 1, 'name': name}
+        initiator_wwn = ['1234567890123456', '1234567890123457']
+
+        find_mp_device_path_mock.return_value = '/dev/mapper/mpatha'
+
+        connection_info = self.fibrechan_connection(vol, location,
+                                                    initiator_wwn)
+        if side_effect and not ignore_exc:
+            self.assertRaises(exception.ExceptionChainer,
+                              self.connector.disconnect_volume,
+                              connection_info['data'], devices['devices'][0],
+                              force=True, ignore_errors=ignore_exc)
+        else:
+            self.connector.disconnect_volume(connection_info['data'],
+                                             devices['devices'][0],
+                                             force=True,
+                                             ignore_errors=ignore_exc)
+
+        flush_mock.assert_called_once_with(
+            find_mp_device_path_mock.return_value)
+
+        expected = [
+            mock.call(f'/dev/disk/by-path/pci-0000:05:00.2-fc-0x{wwn}-lun-1')
+            for wwn in initiator_wwn]
+        if side_effect:
+            del_map_mock.assert_called_once_with('dm-1')
+            expected.append(mock.call(find_mp_device_path_mock.return_value))
+        else:
+            del_map_mock.assert_not_called()
+
+        remove_mock.assert_called_once_with(connection_info['data'],
+                                            devices['devices'],
+                                            devices['devices'][0],
+                                            True, mock.ANY)
+        self.assertEqual(len(expected), realpath_mock.call_count)
+        realpath_mock.assert_has_calls(expected)
