@@ -759,26 +759,6 @@ loop0                                     0"""
         self.assertEqual("0", info['devices'][1]['id'])
         self.assertEqual("3", info['devices'][1]['lun'])
 
-    def test_get_device_size(self):
-        mock_execute = mock.Mock()
-        self.linuxscsi._execute = mock_execute
-        size = '1024'
-        mock_execute.return_value = (size, None)
-
-        ret_size = self.linuxscsi.get_device_size('/dev/fake')
-        self.assertEqual(int(size), ret_size)
-
-        size = 'junk'
-        mock_execute.return_value = (size, None)
-        ret_size = self.linuxscsi.get_device_size('/dev/fake')
-        self.assertIsNone(ret_size)
-
-        size_bad = '1024\n'
-        size_good = 1024
-        mock_execute.return_value = (size_bad, None)
-        ret_size = self.linuxscsi.get_device_size('/dev/fake')
-        self.assertEqual(size_good, ret_size)
-
     def test_multipath_reconfigure(self):
         self.linuxscsi.multipath_reconfigure()
         expected_commands = ['multipathd reconfigure']
@@ -792,7 +772,7 @@ loop0                                     0"""
 
     @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_size')
+    @mock.patch('os_brick.utils.get_device_size')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
     def test_extend_volume_no_mpath(self, mock_device_info,
                                     mock_device_size,
@@ -822,7 +802,7 @@ loop0                                     0"""
 
     @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_size')
+    @mock.patch('os_brick.utils.get_device_size')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
     def test_extend_volume_with_mpath(self, mock_device_info,
                                       mock_device_size,
@@ -854,7 +834,7 @@ loop0                                     0"""
     @mock.patch.object(linuxscsi.LinuxSCSI, '_multipath_resize_map')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_size')
+    @mock.patch('os_brick.utils.get_device_size')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
     def test_extend_volume_with_mpath_fail(self, mock_device_info,
                                            mock_device_size,
@@ -892,7 +872,7 @@ loop0                                     0"""
     @mock.patch.object(linuxscsi.LinuxSCSI, '_multipath_resize_map')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_size')
+    @mock.patch('os_brick.utils.get_device_size')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
     def test_extend_volume_with_mpath_pending(self, mock_device_info,
                                               mock_device_size,
@@ -931,7 +911,7 @@ loop0                                     0"""
     @mock.patch.object(linuxscsi.LinuxSCSI, '_multipath_resize_map')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'find_multipath_device_path')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_scsi_wwn')
-    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_size')
+    @mock.patch('os_brick.utils.get_device_size')
     @mock.patch.object(linuxscsi.LinuxSCSI, 'get_device_info')
     def test_extend_volume_with_mpath_timeout(self, mock_device_info,
                                               mock_device_size,
@@ -1319,6 +1299,64 @@ loop0                                     0"""
         self.linuxscsi.multipath_del_path('/dev/sda')
         self.assertEqual(['multipathd del path /dev/sda'], self.cmds)
 
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name', return_value=None)
+    def test_multipath_del_map_not_present(self, name_mock):
+        self.linuxscsi.multipath_del_map('dm-7')
+        self.assertEqual([], self.cmds)
+        name_mock.assert_called_once_with('dm-7')
+
+    @mock.patch.object(linuxscsi.LinuxSCSI, '_execute')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name', return_value=None)
+    def test_multipath_del_map(self, name_mock, exec_mock):
+        exec_mock.side_effect = [putils.ProcessExecutionError, None]
+
+        mpath_name = '3600d0230000000000e13955cc3757800'
+        name_mock.side_effect = [mpath_name, mpath_name, None]
+        self.linuxscsi.multipath_del_map('dm-7')
+
+        self.assertEqual(2, exec_mock.call_count)
+        exec_mock.assert_has_calls(
+            [mock.call('multipathd', 'del', 'map', mpath_name,
+                       run_as_root=True, timeout=5,
+                       root_helper=self.linuxscsi._root_helper)] * 2)
+        self.assertEqual(3, name_mock.call_count)
+        name_mock.assert_has_calls([mock.call('dm-7')] * 3)
+
+    @mock.patch.object(linuxscsi.LinuxSCSI, '_execute')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name')
+    def test_multipath_del_map_retries_cmd_fails(self, name_mock, exec_mock):
+        exec_mock.side_effect = putils.ProcessExecutionError
+        mpath_name = '3600d0230000000000e13955cc3757800'
+        name_mock.return_value = mpath_name
+        self.assertRaises(putils.ProcessExecutionError,
+                          self.linuxscsi.multipath_del_map, 'dm-7')
+
+        self.assertEqual(3, exec_mock.call_count)
+        exec_mock.assert_has_calls(
+            [mock.call('multipathd', 'del', 'map', mpath_name,
+                       run_as_root=True, timeout=5,
+                       root_helper=self.linuxscsi._root_helper)] * 3)
+
+        self.assertEqual(3, name_mock.call_count)
+        name_mock.assert_has_calls([mock.call('dm-7')] * 3)
+
+    @mock.patch.object(linuxscsi.LinuxSCSI, '_execute')
+    @mock.patch.object(linuxscsi.LinuxSCSI, 'get_dm_name')
+    def test_multipath_del_map_retries_remains(self, name_mock, exec_mock):
+        mpath_name = '3600d0230000000000e13955cc3757800'
+        name_mock.return_value = mpath_name
+        self.assertRaises(exception.BrickException,
+                          self.linuxscsi.multipath_del_map, 'dm-7')
+
+        self.assertEqual(3, exec_mock.call_count)
+        exec_mock.assert_has_calls(
+            [mock.call('multipathd', 'del', 'map', mpath_name,
+                       run_as_root=True, timeout=5,
+                       root_helper=self.linuxscsi._root_helper)] * 3)
+
+        self.assertEqual(6, name_mock.call_count)
+        name_mock.assert_has_calls([mock.call('dm-7')] * 6)
+
     @ddt.data(('/dev/sda', '/dev/sda', False, True, None),
               # This checks that we ignore the was_multipath parameter if it
               # doesn't make sense (because the used path is the one we are
